
CGameManiaPlanet@ appMP;
CGameManiaPlanetScriptAPI@ scriptAPI;
CGameDisplaySettingsWrapper::EDisplayMode currentDisplayMode;
bool autoSwitchDisabledUntilMouseOutside;
bool menuIsOpen = false;

void Main() {
    @appMP = cast<CGameManiaPlanet>(GetApp());
    @scriptAPI = appMP.ManiaPlanetScriptAPI;	

    LoadSettings();
    scriptAPI.DisplaySettings_Unload();
}


bool IsMouseOutsideGame() {
    vec2 pos = UI::GetMousePos();
    if (pos.x < 0 || pos.y < 0) {
        return true;
    }
    if (pos.x > Draw::GetWidth() || pos.y > Draw::GetHeight()) {
        return true;
    }

    return false;
}

void Render() {
    if (Setting_Autoswitch && !menuIsOpen) {
        bool mouseIsOutsideGame = IsMouseOutsideGame();
        if (mouseIsOutsideGame && autoSwitchDisabledUntilMouseOutside) {
            autoSwitchDisabledUntilMouseOutside = false;
        }
        // when the mouse has been outside and we switch, disable switching until it has been outside again
        if (currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::FullscreenExclusive && mouseIsOutsideGame) {
            LoadAndSetWindowedBorderless();
            autoSwitchDisabledUntilMouseOutside = true;

        }
        else if (currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::WindowedFull && !mouseIsOutsideGame && !autoSwitchDisabledUntilMouseOutside) {
            LoadAndSetFullscreen();
        }
    }

    if (!Setting_WindowVisible) {
        return;
    }

    if (Setting_HideWithOverlay && !UI::IsOverlayShown()) {
        return;
    }

    if (!UI::IsGameUIVisible()) {
        return;
    }

    int windowFlags = UI::WindowFlags::NoCollapse | UI::WindowFlags::AlwaysAutoResize;

    if (!UI::IsOverlayShown()) {
        windowFlags |= UI::WindowFlags::NoMove | UI::WindowFlags::NoTitleBar;
    }

    if (UI::Begin("Display Mode", Setting_WindowVisible, windowFlags)){
        float fullscreenColor = 0.6f;
        float windowedColor = 0.6f;
        float windowedBorderlessColor = 0.6f;

        if (currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::FullscreenExclusive)
        {
            fullscreenColor = 0.4f;
        }
        else if (currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::Windowed)
        {
            windowedColor = 0.4f;
        }
        else if (currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::WindowedFull)
        {
            windowedBorderlessColor = 0.4f;
        }

        if (Setting_ShowFullscreen && UI::ButtonColored("Fullscreen", fullscreenColor)) {
            Setting_Autoswitch = false;
            LoadAndSetFullscreen();
        }
        else if (Setting_ShowWindowed && UI::ButtonColored("Windowed", windowedColor)) {
            Setting_Autoswitch = false;
            LoadAndSetWindowed();
        }
        else if (Setting_ShowWindowedBorderless && UI::ButtonColored("Windowed Borderless", windowedBorderlessColor)) {
            Setting_Autoswitch = false;
            LoadAndSetWindowedBorderless();
        }

        Setting_Autoswitch = UI::Checkbox("Autoswitch", Setting_Autoswitch);
        
    }
    UI::End();
}

void RenderMenu() {
    bool disabled = !Setting_ShowFullscreen && !Setting_ShowWindowed && !Setting_ShowWindowedBorderless;
    if (disabled)
    {
        return;
    }

    if (UI::BeginMenu("\\$268" + Icons::Desktop + "\\$z Display Mode Switcher"))
    {
        menuIsOpen = true;
        LoadSettings();

        if (UI::MenuItem("Show Window", "", Setting_WindowVisible)) {
            Setting_WindowVisible = !Setting_WindowVisible;

            menuIsOpen = false;
        }

        if (UI::MenuItem("Autoswitch", "", Setting_Autoswitch)) {
            Setting_Autoswitch = !Setting_Autoswitch;
            menuIsOpen = false;
        }

        UI::Separator();

        if (Setting_ShowFullscreen && UI::MenuItem("Fullscreen", "", currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::FullscreenExclusive)) {
            Setting_Autoswitch = false;
            SetFullscreen();
            menuIsOpen = false;
        }

        if (Setting_ShowWindowed && UI::MenuItem("Windowed", "", currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::Windowed)) {
            Setting_Autoswitch = false;
            SetWindowed();
            menuIsOpen = false;
        }

        if (Setting_ShowWindowedBorderless && UI::MenuItem("Windowed Borderless", "", currentDisplayMode == CGameDisplaySettingsWrapper::EDisplayMode::WindowedFull)) {
            Setting_Autoswitch = false;
            SetWindowedBorderless();
            menuIsOpen = false;
        }

        scriptAPI.DisplaySettings_Unload();

        UI::EndMenu();
    }
    else {
        menuIsOpen = false;
    }
}

void LoadAndSetFullscreen() {
    scriptAPI.DisplaySettings_LoadCurrent();
    SetFullscreen();
    scriptAPI.DisplaySettings_Unload();
}

void LoadAndSetWindowed() {
    scriptAPI.DisplaySettings_LoadCurrent();
    SetWindowed();
    scriptAPI.DisplaySettings_Unload();
}

void LoadAndSetWindowedBorderless() {
    scriptAPI.DisplaySettings_LoadCurrent();
    SetWindowedBorderless();
    scriptAPI.DisplaySettings_Unload();
}

void SetFullscreen() {
    if (Setting_ShowNotifications) {
        UI::ShowNotification("Display Mode Switcher", "Switched to fullscreen");
    }
    
    currentDisplayMode = CGameDisplaySettingsWrapper::EDisplayMode::FullscreenExclusive;
    scriptAPI.DisplaySettings.DisplayMode = CGameDisplaySettingsWrapper::EDisplayMode::FullscreenExclusive;
    scriptAPI.DisplaySettings_Apply();
}

void SetWindowed() {
    if (Setting_ShowNotifications) {
        UI::ShowNotification("Display Mode Switcher", "Switched to windowed");
    }
    currentDisplayMode = CGameDisplaySettingsWrapper::EDisplayMode::Windowed;
    scriptAPI.DisplaySettings.DisplayMode = CGameDisplaySettingsWrapper::EDisplayMode::Windowed;
    scriptAPI.DisplaySettings_Apply();
}

void SetWindowedBorderless() {
    if (Setting_ShowNotifications) {
        UI::ShowNotification("Display Mode Switcher", "Switched to windowed borderless");
    }
    currentDisplayMode = CGameDisplaySettingsWrapper::EDisplayMode::WindowedFull;
    scriptAPI.DisplaySettings.DisplayMode = CGameDisplaySettingsWrapper::EDisplayMode::WindowedFull;
    scriptAPI.DisplaySettings_Apply();
}

void LoadSettings() {
    scriptAPI.DisplaySettings_LoadCurrent();
    currentDisplayMode = scriptAPI.DisplaySettings.DisplayMode;
}

